#' Converts the HTTK-Pop population data table to a table of the parameters
#' needed by HTTK, for a specific chemical.
#' 
#' Takes the data table generated by \code{httkpop_generate}, and converts it
#' to the corresponding table of HTTK model parameters for a specified chemical
#' and HTTK model.
#' 
#' 
#' @param indiv_dt A data table generated by \code{httkpop_generate()}.
#' @param chemcas The CAS number of one of the HTTK chemicals (see
#' \code{\link[httk]{get_cheminfo}}). Defaults to NULL. 
#' @param parameters A list of chemical-specific model parameters containing at
#' least Funbound.plasma, Clint, and Fhep.assay.correction, otherwise defaults
#' to NULL.
#' @param model One of the HTTK models: "1compartment", "3compartmentss",
#' "3compartment", or "pbtk".
#' @param poormetab TRUE (include poor metabolizers) or FALSE (exclude poor
#' metabolizers)
#' @param fup.censored.dist Logical. Whether to draw \code{Funbound.plasma} from a
#' censored distribution or not.
#' @param fup.meas.cv Coefficient of variation of distribution of measured
#' \code{Funbound.plasma} values. 
#' @param clint.meas.cv Coefficient of variation of distribution of measured 
#' \code{Clint} values.
#' @param fup.pop.cv Coefficient of variation of distribution of population
#' \code{Funbound.plasma} values.
#' @param clint.pop.cv Coefficient of variation of distribution of population
#' \code{Clint} values.
#' @param fup.lod The average limit of detection for \code{Funbound.plasma}. if
#' \code{fup.censor == TRUE}, the \code{Funbound.plasma} distribution will be
#' censored below \code{lod/2}. Default value is 0.01.
#' @param adjusted.Funbound.plasma Uses adjusted Funbound.plasma when set to
#' TRUE.
#' @param regression Whether or not to use the regressions in calculating
#' partition coefficients.
#' @param well.stirred.correction If TRUE (default) then the well-stirred
#' correction (Rowland et al., 1973) is used in the calculation of hepatic
#' clearance for the models that do not include flows for first-pass metabolism
#' (currently, 1compartment and 3compartmentss). This assumes clearance
#' relative to amount unbound in whole blood instead of plasma, but converted
#' for use with plasma concentration.
#' @param restrictive.clearance Protein binding not taken into account (set to
#' 1) in liver clearance if FALSE (default TRUE).
#' @param clint.pvalue.threshold Hepatic clearance for chemicals where the in
#' vitro clearance assay result has a p-values greater than the threshold are
#' set to zero.
#' @param concentration Blood, plasma, or tissue concentration.
#' @return A data.table whose columns correspond to the parameters of the HTTK
#' model specified in \code{model}, and whose rows correspond to the
#' individuals (rows) of \code{indiv_dt}. 
#' @author Caroline Ring and John Wambaugh
#' @references Ring, Caroline L., et al. "Identifying populations sensitive to
#' environmental chemicals by simulating toxicokinetic variability."
#' Environment International 106 (2017): 105-118
#' 
#' Rowland, Malcolm, Leslie Z. Benet, and Garry G. Graham. "Clearance concepts
#' in pharmacokinetics." Journal of Pharmacokinetics and Biopharmaceutics 1.2
#' (1973): 123-136. 
#' @examples
#' 
#' set.seed(42)
#' indiv_examp <- httkpop_generate(method="d", nsamp=100)
#' httk_param <- get_httk_params(indiv_dt=indiv_examp, 
#' chemcas="80-05-7", 
#' model="1compartment", 
#' poormetab=TRUE, 
#' fup.censored.dist=TRUE)
#' 
#' @export get_httk_params
get_httk_params <- function(indiv_dt,
                            chemcas=NULL,
                            parameters=NULL,
                            model,
                            poormetab,
                            fup.censored.dist=FALSE,
                            fup.meas.cv=0.4,
                            clint.meas.cv=0.3,
                            fup.pop.cv=0.1,
                            clint.pop.cv=0.1,
                            fup.lod=0.01,
                            adjusted.Funbound.plasma=T,
                            regression=T,
                            well.stirred.correction=T,
                            restrictive.clearance=T,
                            concentration = "plasma",
                            clint.pvalue.threshold=0.05)
{
  
  #First convert to physiological parameters used by HTTK
  indiv_bio <- httkpop_bio(indiv_dt = indiv_dt)
  
  #Next add chemical-specific Funbound.plasma and CLint values
  #Just cbind them together for now
  indiv_fc <- cbind(indiv_bio,
                    draw_fup_clint(this.chem=chemcas,
                      parameters=parameters,
                      nsamp=nrow(indiv_bio),
                      poormetab=poormetab,
                      fup.meas.cv=fup.meas.cv,
                      clint.meas.cv=clint.meas.cv,
                      fup.pop.cv=fup.pop.cv,
                      clint.pop.cv=clint.pop.cv,
                      fup.censored.dist=fup.censored.dist,
                      fup.lod=fup.lod,
                      adjusted.Funbound.plasma=adjusted.Funbound.plasma,
                      clint.pvalue.threshold=clint.pvalue.threshold))
  
  #Next convert the whole thing to the HTTK parameters for a specified model
  indiv_httk <- convert_httk(indiv.model.bio=indiv_fc, 
                 model=model,
                 this.chem=chemcas,
                 parameters=parameters,
                 adjusted.Funbound.plasma=adjusted.Funbound.plasma,
                 regression=regression,
                 well.stirred.correction=well.stirred.correction,
                 restrictive.clearance=restrictive.clearance,
                 concentration = concentration,
                 clint.pvalue.threshold=clint.pvalue.threshold)
  
  return(indiv_httk)
}
